/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000;

import static java.lang.System.out;
import java.io.File;
import java.io.FileReader;
import java.util.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;
import it.andynaz.config.ConfigMgr;
import it.andynaz.irex.dr1000.document.*;
import it.andynaz.irex.dr1000.util.ColorMgr;

/**
 * Main class.
 * 
 * <p>The scribbles can be taken on a PDF document or with the device built-in
 * notepad. In case of PDF documents, this program creates a new PDF with all
 * its interactive elements, with the scribbles on it.<br/>
 * The device must have the version&nbsp;2.0 of the firmware.</p>
 * 
 * <p>To run the program, run in a terminal one of the following (note that the
 * <code>.jar</code> file name may change, as the version can be appended):</p>
 * <blockquote><pre>
 * java -jar [<i>params</i>] iRexDR1000Merger.jar <i>input_file</i>
 * java -jar [<i>params</i>] iRexDR1000Merger.jar -i <i>input_file</i> <i>options</i></pre></blockquote>
 * <p>in a terminal.<br/>
 * In the first case you specify only the input file (using all the default
 * options), while in the second you specify some other option. In both cases
 * the parameters are optional.</p>
 * 
 * <p>To show a brief guide just use the <code>-h</code> option only.</p>
 * 
 * 
 * <h1>Options</h1>
 * 
 * <p>Options for the program are all optional (except for the input file) and
 * can be specified as:
 * <ol style="padding-left: 40px;">
 *   <li style="list-style-type: decimal;">parameters before the <code>-jar</code> part of the command, in the
 *   form<br/><code>-Dit.andynaz.<b>name</b>=<b>value</b></code></li>
 *   <li style="list-style-type: decimal;">parameters in a separate configuration file, in the form<br/>
 *   <code>it.andynaz.<b>name</b>=<b>value</b></code></li>
 *   <li style="list-style-type: decimal;">options, after the <code>.jar</code> file, in the form<br/>
 *   <code>-<b>name</b> value</code></li>
 * </ol>
 * If the same option is specified more than once, the last value (in the order
 * specified above) is set.<br/>
 * In this way, an alias for the command or a batch file can be created on the
 * system specifying the default options, which can be overwritten on the
 * fly.<br/>
 * For instance, to create an alias to set the location of the jar file and the
 * default color, do:<br/>
 * on a Linux machine:
 * <blockquote><pre>
 * alias iRexDR1000='java -Dit.andynaz.color=FF0000 -jar ~/bin/iRexDR1000Merger.jar'</pre></blockquote>
 * on a Windows machine:
 * <blockquote><pre>
 * doskey iRexDR1000=java -Dit.andynaz.color=FF0000 -jar "C:\iRexDR1000Manager.jar" $*</pre></blockquote>
 * </p>
 * 
 * <h3>Parameters</h3>
 * <ul>
 *   <li><b>color</b> color of the strokes in the resulting PDF file; the color
 *   is specified as an hex value {@code RRGGBB}</li>
 *   <li><b>config</b> path to the a properties file with the configuration of
 *   the program</li>
 *   <li><b>targetFolder</b> path to the default folder where to save the
 *   files</li>
 * </ul>
 * 
 * <h3>Options</h3>
 * <ul>
 *   <li><b>i</b> target PDF or {@code .note} file in case of a PDF file, this
 *   program look for the {@code metedata.db} file in the same folder</li>
 *   <li><b>o</b> output file (the default one is a file in the current
 *   folder)</li>
 *   <li><b>a</b> action to be performed (the default one is 'merge')</li>
 *   <li><b>c</b> same as 'config' parameter (the default one is a {@code
 *   config.properties} file in the current directory)</li>
 *   <li><b>C</b> same as 'color' parameter (default is the black color)</li>
 * </ul>
 * 
 * <h3>Actions</h3>
 * <p>The action (with the <code>-a</code> option) tell the program what to
 * do:</p>
 * <ul>
 *   <li><b>merge</b> default action: merge the scribbles with the PDF in a new
 *   PDF file or creates a new PDF file with the note (default output file is a
 *   PDF file in the current folder)</li>
 *   <li><b>png</b> create a PNG image for each page with the scribbles</li>
 *   <li><b>format</b> create a text file with the scribbles data formatted
 *   (default output file is a {@code .dat} file in the current folder)</li>
 * </ul>
 *
 * @author andynaz
 * @version 2013/10/01
 */
public class Manager {

   /**
	 * Takes the arguments from the command line and start the program.
	 * 
	 * @param args the command line arguments
	 * @throws Exception in case of errors
    */
   public static void main(String[] args) throws Exception {
		Utils.printHeader();
		
		// check arguments: at least one
      if (args.length==0) {
			out.println("error: no arguments");
			out.println("use the -h option for help");
			System.exit(1);
		}
		
		// help
		if (args.length==1 && "-h".equals(args[0])) {
			Utils.printHelp();
			System.exit(0);
		}
		
		// ---------- configuration ----------------------------------------------
		ConfigMgr.init();
		
		// configuration file
		ConfigMgr.setConfigParam("config", Utils.getInitParam("c", args));
		String configPath = ConfigMgr.getConfigParam("config");
		configPath = Utils.isEmpty(configPath) ? "config.properties" : configPath;
		File configFile = new File(configPath);
		if (configFile.exists()) {
			Properties p = new Properties();
			p.load(new FileReader(configFile));
			ConfigMgr.init(p);
		}
		
		// logger
		Logger logger = LoggerMgr.getLogger("main ");
		
		logger.log(Arrays.toString(args), Level.FINE);
		logger.log("config:"+ConfigMgr.getConfigParam("config"), Level.DEBUG);
		
		// stroke color
		ColorMgr.setColor(Utils.getInitParam("C", args));
		logger.log("stroke color: "+ColorMgr.getColor(), Level.DEBUG);
		
		
		// check if the program has been invoked with the only-one-argument syntax
		boolean oneArgOnly = args.length==1;
		
		
		// ---------- execution data ---------------------------------------------
		// --- input file
		String inputFilePath = oneArgOnly ? args[0] : Utils.getInitParam("i", args);
		if (inputFilePath==null) {
			logger.log("error: no input file", Level.ERROR);
			out.println("error: no input file");
			out.println("use the -h option for help");
			return;
		}
		logger.log("input file: "+inputFilePath, Level.DEBUG);
		File inputFile = new File(inputFilePath);
		if (!inputFile.exists()){
			logger.log("error: input file does not exists", Level.ERROR);
			return;
		}
		Document doc = Document.getDocument(inputFile);
		
		// --- action
		String action = oneArgOnly ? "merge" : Utils.getInitParam("a", args);
		if (action==null) action="merge";
		if ("strictMerge".equals(action)){
			ConfigMgr.setConfigParam("strictMerge", "true");
			action="merge";
		}
		logger.log("action: "+action, Level.DEBUG);
		
		// --- output
		String outputPath = Utils.getInitParam("o", args);
		String outputFolder = ConfigMgr.getConfigParam("targetFolder");
		if (!Utils.isEmpty(outputPath)){
			File outputFile = new File(outputPath);
			if (outputFile.isDirectory()) {
				logger.log("error: output file path is not a folder", Level.ERROR);
				out.println("error: output file path is not a folder");
				System.exit(1);
			}
			
		} else if (!Utils.isEmpty(outputFolder)) {
			File outputFile = new File(outputFolder);
			if (!outputFile.isDirectory()) {
				logger.log("error: output folder path is not a folder", Level.ERROR);
				out.println("error: output folder path is not a folder");
				System.exit(1);
			}
			logger.log("specified output folder", Level.DEBUG);
			outputPath = "png".equals(action) ? "." : outputFolder+File.separatorChar+doc.getDefaultFileName()+"__NOTED.pdf";
			
		} else {
			outputPath = "png".equals(action) ? "." : doc.getDefaultFileName()+"__NOTED.pdf";
		}
		logger.log("output file: "+outputPath, Level.DEBUG);
		
		
		// ----------  ------------------------------------------------------
		
		if ("merge".equals(action)){
			doc.merge(outputPath);
		} else if ("format".equals(action)){
			doc.format(outputPath);
		} else if ("png".equals(action)){
			doc.exportToPNG(outputPath);
		} else {
			logger.log("error: unknown action", Level.ERROR);
			out.println("error: unknown action");
			out.println("use the -h option for help");
			return;
		}
   }
	
}
