/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import java.awt.Color;
import java.io.*;
import java.util.List;
import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import it.andynaz.log.*;
import it.andynaz.irex.dr1000.scribble.*;

/**
 * Represent a note taken with the iRex DR1000 built-in notepad.
 *
 * @author andynaz
 * @version 2013/10/01
 */
public class Note extends Document{
	
	/**
	 * Creates a new Note object.
	 * 
	 * @param db database file
	 * 
	 * @throws IllegalArgumentException il the {@code db} file does not exists or
	 * is not a file
	 */
	public Note(File db){
		super(db);
	}


	@Override
	protected String getFileName() {
		return ".";
	}
	
	/**
	 * Return the name of the database file, without the extension.
	 * 
	 * @return the name of the database file, without the extension
	 */
	@Override
	public String getDefaultFileName(){
		String fileName = this.db.getName();
		return (fileName.indexOf('.')==-1 ? fileName : fileName.substring(0, fileName.lastIndexOf('.')));
	}

	/**
	 * Creates a PDF file with the scribbles in it.
	 * 
	 * The generated PDF file has pages in B5 format. To create the final PDF, it
	 * first creates an empty PDF with the needed number of pages; after the
	 * merge, the empty PDF is deleted.
	 * 
	 * @param outputFilePath the path to the output PDF file; if it is {@code
	 * null} or empty, the default one returned by {@link #getDefaultFileName()}
	 * is used
	 * @param color strokes' color
	 * 
	 * @see <a href="http://en.wikipedia.org/wiki/Paper_size#B_series">B<i>n</i> paper format</a>
	 */
	@Override
	public void merge(String outputFilePath, Color color){
		Logger logger = LoggerMgr.getLogger("merge");
		
		try{
			ScribbleMgr sm = getScribbleMgr();

			int idFile = sm.getFileId(getFileName());
			if (idFile==0) return;
			
			List<Integer> scribbledPages = sm.getScribbledPages(idFile);
			int nPages = scribbledPages.size();
			logger.log("pag found: "+nPages);
			
			// creates an empty file as a base to merge the scribbles
			File emptyFile = File.createTempFile("empty"+nPages, ".pdf");
			emptyFile.deleteOnExit();
			com.lowagie.text.Document document = new com.lowagie.text.Document(PageSize.B5);
			PdfWriter w = PdfWriter.getInstance(document, new FileOutputStream(emptyFile));
			document.open();
			for (int i=0; i<nPages; i++){
				document.newPage();
				w.setPageEmpty(false);
			}
			document.close();
			
			
			PdfReader reader = new PdfReader(emptyFile.getPath());
			PdfStamper stamp = new PdfStamper(reader, new FileOutputStream(outputFilePath));
			for (int pageNum=1; pageNum<=reader.getNumberOfPages(); pageNum++){
				if (scribbledPages.contains(pageNum)){
					byte[] blob = sm.getScribbles(pageNum, idFile);
					Page p = Parser.build(blob);
					PdfContentByte cbOver = stamp.getOverContent(pageNum);
					Rectangle rectCropBox = reader.getCropBox(pageNum);
					int nRotation = reader.getPageRotation(pageNum);
					while (nRotation>0) {
						rectCropBox = rectCropBox.rotate();
						nRotation -= 90;
					}
					if (p!=null)
						p.drawStrokes(cbOver, rectCropBox.left(), rectCropBox.top(), color);
				}
			}
			stamp.close();
			sm.closeConnection();
			emptyFile.delete();
		} catch(Exception e){
			LoggerMgr.getLogger("merge").log(e, Level.WARNING);
		}
	}
	
}
