/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.scribble;

import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 * Parses the blob to create the pages with the scribbles.
 * 
 * The scribbles are organized as a list of {@link Stroke} objects, which have a
 * list of {@link Stroke.Point} objects.
 * 
 * <p>Scribbles are stored in the {@code annotations.data} column (see
 * documentation of class {@link it.andynaz.irex.dr1000.document.ScribbleMgr}
 * for information about the structure of the database), as a binary blob with
 * the following structure.
 * The not-used parts are represented in gray (in this case the meaning is not
 * sure) and the bytes in the examples  are represented as a couple of
 * hexadecimal digits. Note that the bytes are stored in a reversed order, so
 * that the pair of bytes "01 00" has to be read as the hex number "0001", which
 * corresponds to 1 (and not to 256)</p>
 * 
 * <p>The first 40 bytes are the ones related to the page data:</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th> <th>meaning</th>           <th>example</th></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>version</td>           <td>01 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>orientation</td>       <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>8</td>      <td></td>     <td>page_pixel</td>        <td>00 00 00 00 00 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_r</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_g</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_b</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>ul_x</td>              <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>ul_y</td>              <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>lr_x</td>              <td>D7 04 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>lr_y</td>              <td>DA 06 00 00</td></tr>
 * <tr>                        <td>4</td>      <td>int</td>  <td>number of strokes</td> <td>A3 00 00 00</td></tr>
 * </table>
 * <p>Only the last one is actually used, as it indicates the number of strokes
 * in the page.</p>
 * 
 * <p>After that 40 bytes, there are the strokes.<br/>
 * Each stroke begin with a 24-bytes sequence, representing the properties of a
 * stroke, and the list of points.<br/>
 * The 24 bytes are strucutred according to the following schema:</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th>  <th>meaning</th>          <th>example</th>     </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>color</td>            <td>03 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>float</td> <td>zoom</td>             <td>D7 A0 89 42</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>layer</td>            <td>00 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>shape</td>            <td>00 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>width</td>            <td>01 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>number of points</td> <td>35 00 00 00</td> </tr>
 * </table>
 * 
 * <p>Each point is represented as a 12-bytes blocks:</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th>  <th>meaning</th>      <th>example</th>     </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>x coordinate</td> <td>48 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>y coordinate</td> <td>6C 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>pressure</td>     <td>00 00 00 00</td> </tr>
 * </table>
 * 
 * <p>After the sequence of points, another stroke begins (or the end of the
 * blob is reached).</p>
 *
 * @author andynaz
 * @version 2013/09/23
 */
public class Parser {

	//hides the constructor
	private Parser(){}

	/**
	 * Create a Page from the blob.
	 *
	 * This method also creates all the {@link Stroke} objects inside the {@link
	 * Page} object.
	 *
	 * @param blob the blob, as taken from the SQLite database file
	 * @return a Page with all the scribbles in the blob, or {@code null} if the
	 * {@code blob} array is empty or {@code null}
	 * 
	 * @throws IOException in an error occours while reading the blob
	 */
	public static Page build(byte[] blob) throws IOException {
		Page p = null;
		if (blob == null || blob.length==0)
			return p;
		
		p = new Page();
		ByteArrayInputStream bIn = new ByteArrayInputStream(blob);
		safeSkip(bIn, 36);
		int nStroke = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());
		for (int c = 0; c < nStroke; c++) {
			Stroke s = new Stroke();
			safeSkip(bIn, 4);
			float zoom = Util.get4ByteFloat(bIn.read(), bIn.read(), bIn.read(), bIn.read());
			s.setZoom(zoom);
			safeSkip(bIn, 8);
			int width = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());
			s.setWidth(width);
			int nPoint = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());
			for (int j = 0; j < nPoint; j++) {
				int x = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());
				int y = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());
				s.addPoint(x, y);
				safeSkip(bIn, 4);
			}
			p.addStroke(s);
		}
		assert bIn.read()==-1 : "the blob is not ended";
		return p;
	}
	
	/**
	 * Skips {@code l} bytes from the ByteArrayInputStream {@code bIn}.
	 * 
	 * After skipping, check if the actually skipped bytes are exactly {@code l};
	 * if not, throws an IOException (the number of bytes actually skipped is in
	 * the IOException message).
	 * 
	 * @param bIn stream from whom skip the bytes
	 * @param l number of bytes to skip
	 * 
	 * @throws IOException if the method skipped less than {@code l} bytes or if
	 * an IO error occours
	 */
	private static void safeSkip(ByteArrayInputStream bIn, long l) throws IOException{ //TODO inserirlo nelle andyLib?
		long skipped = bIn.skip(l);
		if (skipped!=l)
			throw new IOException("skipped "+skipped+" bytes out of "+l);
	}
	
}
