/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.scribble;

import java.awt.Color;
import java.util.*;
import com.lowagie.text.pdf.PdfContentByte;
import it.andynaz.irex.dr1000.util.ColorMgr;

/**
 * A stroke, made up of various points.
 * 
 * Points can be added via the {@link #addPoint(int, int)} method and are
 * memorized in a ordered list.<br/>
 * A stroke has also a width and a zoom level.
 *
 * @author andynaz
 * @version 2013/09/22
 */
public class Stroke {
	
	/** List of points. */
	private List<Point> points = new ArrayList<Point>();
	
	/** Zoom level. */
	private float zoom;
	
	/** Width of the stroke. */
	private int width;
	

	/**
	 * Create a stroke.
	 */
	public Stroke(){ }
	

	/**
	 * Returns the width of this stroke.
	 * 
	 * @return the width of this stroke
	 */
	public int getWidth(){
		return width;
	}
	
	/**
	 * Set the width of this stroke.
	 * 
	 * @param width the width of the stroke (must be positive)
	 * 
	 * @throws IllegalArgumentException if {@code width} is not positive
	 */
	public void setWidth(int width) {
		if (width<=0)
			throw new IllegalArgumentException("non-positive width");
		this.width = width;
	}

	/**
	 * Set the zoom level of this stroke.
	 * 
	 * @param zoom the zoom level of the stroke (must be positive)
	 * 
	 * @throws IllegalArgumentException if {@code zoom} is not positive
	 */
	public void setZoom(float zoom) {
		if (zoom<=0)
			throw new IllegalArgumentException("non-positive zoom level");
		this.zoom = zoom;
	}

	/**
	 * Add a point to this stroke.
	 * 
	 * the point is specified by its coordinates.
	 *
	 * @param x 'x' coordinate of the point to add
	 * @param y 'y' coordinate of the point to add
	 */
	public void addPoint(int x, int y){
		Point p = new Point(x, y);
		points.add(p);
	}
	
	/**
	 * Returns the list of points in this page.
	 * 
	 * @return the list of points in this page
	 */
	public List<Point> getPoints(){
		return points;
	}

	/**
	 * Draw the stroke.
	 * 
	 * Alla the points in this stroke are drawn in the order htey have been
	 * insertend in the stroke.
	 * 
	 * <p>When being drawn on the PDF, all the points of the stroke are scaled
	 * and adapted for the iText library, which uses a different coordinate
	 * system as the one used by the iPDF library, as shown in the following
	 * picture.<p>
	 * 
	 * <p><img src="doc-files/coordinate.png" /></p>
	 * 
	 * <p>At first, the poinys' coordinates are scaled using the zoom factor as
	 * (as explained <a target="_balnk"
	 * href="http://www.mobileread.com/forums/showpost.php?p=868622&postcount=13"
	 * >here</a>):
	 * <pre>
	 *          x                1            48 x
	 * x' = ---------- * ----------------- = ------
	 *       zoom/100     160/72 (1-1/16)     zoom
	 * 
	 *          y                1            48 y
	 * y' = ---------- * ----------------- = ------
	 *       zoom/100     160/72 (1-1/16)     zoom</pre>
	 * so the zoom factor has to be set with the {@link #setZoom(float)} method
	 * before calling the {@link #draw(PdfContentByte, float, float)}
	 * method).<br/>
	 * After scaling, the coordinates are converted to the iText coordinates
	 * system using the formulas
	 * <pre>
	 * x' = x<sub>a</sub> + x
	 * 
	 * y' = y<sub>a</sub> - y</pre>
	 * where <code>x<sub>a</sub></code> and <code>y<sub>a</sub></code> are the
	 * arguments of the method.</p>
	 * 
	 * @param cb content of the PDF
	 * @param xAdjust value to correct the 'x' coordinate of all the points in
	 * this stroke
	 * @param yAdjust value to correct the 'y' coordinate of all the points in
	 * this stroke
	 * 
	 * @throws ArithmeticException if the {@link #setZoom(float)} method has not
	 * been called
	 */
	void draw(PdfContentByte cb, float xAdjust, float yAdjust, Color c) {
		if (points.size()<2)
			return;
      
		// also the line width is scaled
		cb.setLineWidth( (720*width)/(zoom*15) );
		cb.setColorStroke(c);

		float x,y;
		x = xAdjust + points.get(0).getScaledX();
		y = yAdjust - points.get(0).getScaledY();
		cb.moveTo(x, y);
		for (int pointNumber = 1; pointNumber<points.size(); pointNumber++) {
			x = xAdjust + points.get(pointNumber).getScaledX();
			y = yAdjust - points.get(pointNumber).getScaledY();
			cb.lineTo(x, y);
		}

		cb.stroke();
	}
	
	/**
	 * Draw the stroke.
	 * 
	 * The stroke is drawn using the default color for the strokes (as specified
	 * in the {@link ColorMgr} class).
	 * 
	 * @param cb content of the PDF
	 * @param xAdjust value to correct the 'x' coordinate of all the points in
	 * this stroke
	 * @param yAdjust value to correct the 'y' coordinate of all the points in
	 * this stroke
	 * 
	 * @throws ArithmeticException if the {@link #setZoom(float)}} method has not
	 * been called
	 * 
	 * @see #draw(PdfContentByte, float, float, Color)
	 */
	void draw(PdfContentByte cb, float xAdjust, float yAdjust) {
		draw(cb, xAdjust, yAdjust, ColorMgr.getColor());
	}

	/**
	 * A point.
	 *
	 * A point is made of two int coordinates, 'x' and 'y'.
	 */
	public class Point {
		private int x;
		private int y;
		
		/**
		 * Creates a new point.
		 * 
		 * @param x 'x' coordinate of the point
		 * @param y 'y' coordinate of the point
		 */
		Point(int x, int y) {
			this.x = x;
			this.y = y;
		}
		
		public int getX(){
			return this.x;
		}
		
		public int getY(){
			return this.y;
		}
		
		/**
		 * Return the scaled value of the 'y' coordinate of this point.
		 * 
		 * The value is calculated considering the zoom level of the stroke.
		 * 
		 * @return the scaled 'y' value of this point
		 */
		public float getScaledX(){
			return 48*this.x/zoom;
		}
		
		/**
		 * Return the scaled value of the 'y' coordinate of this point.
		 * 
		 * The value is calculated considering the zoom level of the stroke.
		 * 
		 * @return the scaled 'y' value of this point
		 */
		public float getScaledY(){
			return 48*this.y/zoom;
		}
	}
	
}
